#include "AlarmingTheme.h"
#include <QPixmap>
#include <QColor>
#include <stdio.h>

AlarmingTheme::AlarmingTheme(QString init_theme_dir)
 : theme_dir(init_theme_dir),
   alarm_background_pixmap(NULL),
   reminder_text_font(NULL),
   reminder_text_qcolor(NULL),
   alarm_background("alarm.png"),
   should_stay_on_top(true),
   should_bypass_x11_wm(true),
   reminder_text_box_left(83),
   reminder_text_box_top(205),
   reminder_text_box_width(198),
   reminder_text_box_height(118),
   reminder_text_font_family("Helvetica"),
   reminder_text_font_point_size(24),
   reminder_text_font_weight(QFont::Bold),
   reminder_text_font_italic(false),
   reminder_text_color("red"),
   ok_button_position_left(31),
   ok_button_position_top(60),
   ok_button_position_width(83),
   ok_button_position_height(38),
   ok_button_text("&OK"),
   snooze_button_position_left(243),
   snooze_button_position_top(56),
   snooze_button_position_width(83),
   snooze_button_position_height(38),
   snooze_button_text("&Snooze"),
   in_element(false) {
}

AlarmingTheme::~AlarmingTheme() {
   delete alarm_background_pixmap;
   delete reminder_text_font;
   delete reminder_text_qcolor;
}

bool
AlarmingTheme::startElement(const QString& namespaceURI,
   const QString& localName,
   const QString& qName,
   const QXmlAttributes& atts) {
   elts.push_back(localName);
   in_element = true;
   ch_accum.truncate(0);
   return true;
}

bool
AlarmingTheme::endElement(const QString& namespaceURI,
   const QString& localName,
   const QString& qName) {

   // OK, at this point, ch_accum has the string with the data we want, and
   // elts is a stack of xml element names that tell us what to do with the
   // data. For forward compatibility, we always just ignore anything we
   // don't recognize. We never generate errors.

   QStringList::Iterator it = elts.begin();
   if ((it != elts.end()) && (*it == "qtmess")) {
      ++it;
      if (it != elts.end()) {
         QString elt1 = *it;
         if (elt1 == "alarm_background") {
            alarm_background = ch_accum;
         } else if (elt1 == "should_stay_on_top") {
            should_stay_on_top = boolVal(ch_accum);
         } else if (elt1 == "should_bypass_x11_wm") {
            should_bypass_x11_wm = boolVal(ch_accum);
         } else if (elt1 == "reminder_text_box") {
            ++it;
            if (it != elts.end()) {
               QString elt2 = *it;
               if (elt2 == "left") {
                  reminder_text_box_left = intVal(ch_accum);
               } else if (elt2 == "top") {
                  reminder_text_box_top = intVal(ch_accum);
               } else if (elt2 == "width") {
                  reminder_text_box_width = intVal(ch_accum);
               } else if (elt2 == "height") {
                  reminder_text_box_height = intVal(ch_accum);
               }
            }
         } else if (elt1 == "reminder_text_font") {
            ++it;
            if (it != elts.end()) {
               QString elt2 = *it;
               if (elt2 == "family") {
                  reminder_text_font_family = ch_accum;
               } else if (elt2 == "point_size") {
                  reminder_text_font_point_size = intVal(ch_accum);
               } else if (elt2 == "weight") {
                  if (ch_accum == "Light") {
                     reminder_text_font_weight = QFont::Light;
                  } else if (ch_accum == "Normal") {
                     reminder_text_font_weight = QFont::Normal;
                  } else if (ch_accum == "DemiBold") {
                     reminder_text_font_weight = QFont::DemiBold;
                  } else if (ch_accum == "Bold") {
                     reminder_text_font_weight = QFont::Bold;
                  } else if (ch_accum == "Black") {
                     reminder_text_font_weight = QFont::Black;
                  }
                  ok_button_position_width = intVal(ch_accum);
               } else if (elt2 == "italic") {
                  reminder_text_font_italic = boolVal(ch_accum);
               }
            }
         } else if (elt1 == "reminder_text_color") {
            reminder_text_color = ch_accum;
         } else if (elt1 == "ok_button_position") {
            ++it;
            if (it != elts.end()) {
               QString elt2 = *it;
               if (elt2 == "left") {
                  ok_button_position_left = intVal(ch_accum);
               } else if (elt2 == "top") {
                  ok_button_position_top = intVal(ch_accum);
               } else if (elt2 == "width") {
                  ok_button_position_width = intVal(ch_accum);
               } else if (elt2 == "height") {
                  ok_button_position_height = intVal(ch_accum);
               }
            }
         } else if (elt1 == "ok_button_text") {
            ok_button_text = ch_accum;
         } else if (elt1 == "snooze_button_position") {
            ++it;
            if (it != elts.end()) {
               QString elt2 = *it;
               if (elt2 == "left") {
                  snooze_button_position_left = intVal(ch_accum);
               } else if (elt2 == "top") {
                  snooze_button_position_top = intVal(ch_accum);
               } else if (elt2 == "width") {
                  snooze_button_position_width = intVal(ch_accum);
               } else if (elt2 == "height") {
                  snooze_button_position_height = intVal(ch_accum);
               }
            }
         } else if (elt1 == "snooze_button_text") {
            snooze_button_text = ch_accum;
         }
      }
   }
   elts.pop_back();
   in_element = false;
   return true;
}

bool
AlarmingTheme::characters(const QString& ch) {
   if (in_element) {
      ch_accum.append(ch);
   }
   return true;
}

QPixmap *
AlarmingTheme::AlarmBackground() {
   if (alarm_background_pixmap == NULL) {
      QString abspat("/");
      if (! alarm_background.startsWith(abspat)) {
         alarm_background.prepend(theme_dir);
      }
      alarm_background_pixmap = new QPixmap(alarm_background);
   }
   return alarm_background_pixmap;
}

bool
AlarmingTheme::ShouldSetPosition() {
   return false;
}

QPoint
AlarmingTheme::Position() {
   return QPoint(0,0);
}

bool
AlarmingTheme::ShouldStayOnTop() {
   return should_stay_on_top;
}

bool
AlarmingTheme::ShouldBypassX11WM() {
   return should_bypass_x11_wm;
}

QRect
AlarmingTheme::ReminderTextBox() {
   return QRect(reminder_text_box_left, reminder_text_box_top,
                reminder_text_box_width, reminder_text_box_height);
}

QFont *
AlarmingTheme::ReminderTextFont() {
   if (reminder_text_font == NULL) {
      reminder_text_font =
         new QFont(reminder_text_font_family, reminder_text_font_point_size,
                   reminder_text_font_weight, reminder_text_font_italic);
   }
   return reminder_text_font;
}

QColor *
AlarmingTheme::ReminderTextColor() {
   if (reminder_text_qcolor == NULL) {
      reminder_text_qcolor = new QColor(reminder_text_color);
   }
   return reminder_text_qcolor;
}

QRect
AlarmingTheme::OKButtonPosition() {
   return QRect(ok_button_position_left, ok_button_position_top,
                ok_button_position_width, ok_button_position_height);
}

QIcon *
AlarmingTheme::OKButtonIcons() {
   return NULL;
}

QString
AlarmingTheme::OKButtonText() {
   return QString(ok_button_text);
}

QRect
AlarmingTheme::SnoozeButtonPosition() {
   return QRect(snooze_button_position_left, snooze_button_position_top,
                snooze_button_position_width, snooze_button_position_height);
}

QIcon *
AlarmingTheme::SnoozeButtonIcons() {
   return NULL;
}

QString
AlarmingTheme::SnoozeButtonText() {
   return QString(snooze_button_text);
}

bool
AlarmingTheme::boolVal(QString s) {
   QString ss = s.trimmed();
   if (ss.length() > 0) {
      QChar c = ss[0];
      if ((c == 't') || (c == 'T') || (c == 'y') || (c == 'Y') || (c == '1')) {
         return true;
      }
   }
   return false;
}

int
AlarmingTheme::intVal(QString s) {
   return s.toInt();
}
