#ifndef AlarmingTheme_h
#define AlarmingTheme_h

#include <QPoint>
#include <QRect>
#include <QString>
#include <QtXml/QXmlDefaultHandler>
#include <QFont>

class QPixmap;
class QColor;
class QIcon;

// The AlarmingWidget gets all its configuration data from an AlarmingTheme
// object (which (FIXME someday) get's its data from an XML config file).
//
class AlarmingTheme : public QXmlDefaultHandler {
public:

   AlarmingTheme(QString init_theme_dir);

   ~AlarmingTheme();

   virtual bool
   startElement(const QString& namespaceURI,
                const QString& localName,
                const QString& qName,
                const QXmlAttributes& atts);

   virtual bool
   endElement(const QString& namespaceURI,
              const QString& localName,
              const QString& qName);

   virtual bool
   characters(const QString& ch);

   // The image and position of the alarm popup

      // Return QPixmap * to be used as the background image. The top level
      // widget also has the mask() of this pixmap applied so that
      // transparent pixels disappear utterly.
      //
   QPixmap *
   AlarmBackground();

      // If true, set the (x,y) position of the alarm widget, if false
      // let Qt or window manager or someone set it at random.
      //
   bool
   ShouldSetPosition();

      // If ShouldSetPosition() is true, use this position for top level
      // alarm widget placement.
      //
   QPoint
   Position();

      // If true, set the widget flag to tell window manager to keep
      // this on top.
      //
   bool
   ShouldStayOnTop();

      // If true set other silly flag that is used to really make the
      // widget stay on top.
      //
   bool
   ShouldBypassX11WM();

   // The text of the reminder drawn in the alarm popup window

      // Rectangle within the AlarmBackground() pixmap where the reminder
      // text should be rendered.
      //
   QRect
   ReminderTextBox();

      // The font to use for the reminder text (this font will be scaled
      // to fit the text into the ReminderTextBox() area).
      //
   QFont *
   ReminderTextFont();

      // The color to use for rendering the reminder text in the alarm window.
      //
   QColor *
   ReminderTextColor();

   // The OK Button

      // The geometry (relative to top level window) of the OK button
      // in the alarm. (Pressing this button makes the alarm go away
      // and never comde back).
      //
   QRect
   OKButtonPosition();

      // The icons for the OK button (or NULL if it is a text button).
      //
   QIcon *
   OKButtonIcons();

      // If OKButtonPixmap() is NULL, then use this text for the button.
      //
   QString
   OKButtonText();

   // The Snooze Button

      // The geometry (relative to top level window) of the Snooze button
      // in the alarm. (Pressing this button makes the alarm go away
      // then come back later (FIXME someday)).
      //
   QRect
   SnoozeButtonPosition();

      // The icons for the Snooze button (or NULL if it is a text button).
      //
   QIcon *
   SnoozeButtonIcons();

      // If SnoozeButtonPixmap() is NULL, then use this text for the button.
      //
   QString
   SnoozeButtonText();

private:

   bool
   boolVal(QString s);

   int
   intVal(QString s);

   QString theme_dir;

   QPixmap * alarm_background_pixmap;
   QFont * reminder_text_font;
   QColor * reminder_text_qcolor;

   QString alarm_background;
   bool should_stay_on_top;
   bool should_bypass_x11_wm;
   int reminder_text_box_left;
   int reminder_text_box_top;
   int reminder_text_box_width;
   int reminder_text_box_height;
   
   QString reminder_text_font_family;
   int reminder_text_font_point_size;
   QFont::Weight reminder_text_font_weight;
   bool reminder_text_font_italic;
   
   QString reminder_text_color;
   int ok_button_position_left;
   int ok_button_position_top;
   int ok_button_position_width;
   int ok_button_position_height;
   
   QString ok_button_text;
   int snooze_button_position_left;
   int snooze_button_position_top;
   int snooze_button_position_width;
   int snooze_button_position_height;
   
   QString snooze_button_text;

   bool in_element;
   QString ch_accum;
   QStringList elts;
};

#endif // AlarmingTheme_h
