#ifndef AlarmingWidget_h
#define AlarmingWidget_h

#include <QWidget>
#include <QStringList>

class AlarmingTheme;
class QPushButton;
class QTimer;

// An AlarmingWidget is a top level widget designed to display an alarm with
// snooze button for use in reminder applications.
//
// It uses a Pixmap as a background for the reminder text and shapes itself
// to the non-transparent widgets for a fancy visual effect you'll be sure
// to notice even if half asleep :-).
//
class AlarmingWidget : public QWidget {

   Q_OBJECT

public:

      // Create a top level widget from the data in "theme" and the lines of
      // text from "reminder".
      //
   AlarmingWidget(
      AlarmingTheme * theme,
      QStringList reminder,
      int init_snooze_ms,
      const char * name = 0);

   ~AlarmingWidget();

private slots:

   void
   doSnooze();

   void
   wakeUp();

protected:

   void
   mousePressEvent(QMouseEvent * e);

   void
   mouseMoveEvent(QMouseEvent * e);

private:

   static Qt::WindowFlags
   WindowFlags(AlarmingTheme * theme);

   void
   DrawReminderText(
      AlarmingTheme * theme,
      QStringList reminder);

   QPushButton * ok_btn;
   QPushButton * snooze_btn;
   QPoint clickPos;

   QTimer * snooze_timer;
   int snooze_ms;
};

#endif // AlarmingWidget_h
