#include "AlarmingTheme.h"
#include "AlarmingWidget.h"
#include "FindTheme.h"
#include "RecordTimestamp.h"

#include <QApplication>
#include <QFile>
#include <QtXml/QXmlSimpleReader>
#include <QtXml/QXmlInputSource>

int
main(int argc, char ** argv) {
   QApplication a(argc, argv);

   QString theme_name("boring");
   QString snooze("5min");
   QString timestamp;
   int snooze_milliseconds;
   int snooze_mul = 1;

   // Skip past program name in arg list

   --argc;
   ++argv;

   // See if we have -theme and/or -snooze args

   while (argc >= 2) {
      QString a(argv[0]);
      if (a == "-theme") {
         theme_name = argv[1];
      } else if (a == "-snooze") {
         snooze = argv[1];
      } else if (a == "-stamp") {
         timestamp = argv[1];
         RecordTimestamp(timestamp);
      } else {
         break;
      }
      argc -= 2;
      argv += 2;
   }

   if (snooze.endsWith("min")) {
      snooze_mul = 60*1000;
      snooze = snooze.left(snooze.length() - 3);
   } else if (snooze.endsWith("sec")) {
      snooze_mul = 1000;
      snooze = snooze.left(snooze.length() - 3);
   } else if (snooze.endsWith("hr")) {
      snooze_mul = 60*60*1000;
      snooze = snooze.left(snooze.length() - 2);
   }
   snooze_milliseconds = snooze.toInt() * snooze_mul;
   if (snooze_milliseconds <= 0) {
      snooze_milliseconds = 5*60*1000;
   }

   QString theme_dir = FindTheme(theme_name);
   if (theme_dir.isNull()) {
      return 2;
   }

   // Reminder text comes from argument list (one arg per line).

   QStringList reminder;
   if (argc <= 0) {
      reminder.append("Ding!");
   } else {
      for (int i = 0; i < argc; ++i) {
         reminder.append(argv[i]);
      }
   }

   // Build the theme

   QString theme_file(theme_dir);
   theme_file.append("qtmess.xml");
   QFile xmlFile(theme_file);
   QXmlInputSource source(&xmlFile);
   QXmlSimpleReader reader;
   AlarmingTheme theme(theme_dir);
   reader.setContentHandler(&theme);
   reader.parse(source);

   // And make the main window and show it.

   AlarmingWidget qtmess(&theme, reminder, snooze_milliseconds, "qtmess");

   qtmess.setWindowTitle("Reminder");
   qtmess.show();
   return a.exec();
}
